@@ -77,6 +77,7 @@ import com.openexchange.mail.filter.json.v2.json.mapper.parser.action.PGPEncrypt
 import com.openexchange.mail.filter.json.v2.json.mapper.parser.action.RedirectActionCommandParser;
 import com.openexchange.mail.filter.json.v2.json.mapper.parser.action.RejectActionCommandParser;
 import com.openexchange.mail.filter.json.v2.json.mapper.parser.action.RemoveFlagActionCommandParser;
+import com.openexchange.mail.filter.json.v2.json.mapper.parser.action.SetActionCommandParser;
 import com.openexchange.mail.filter.json.v2.json.mapper.parser.action.SetFlagActionCommandParser;
 import com.openexchange.mail.filter.json.v2.json.mapper.parser.action.StopActionCommandParser;
 import com.openexchange.mail.filter.json.v2.json.mapper.parser.action.VacationActionCommandParser;
@@ -191,6 +192,8 @@ public class MailFilterJSONActivator extends AJAXModuleActivator {
         registry.register(ActionCommand.Commands.PGP_ENCRYPT.getJsonName(), new PGPEncryptActionCommandParser(this));
 
         registry.register(SimplifiedAction.COPY.getCommandName(), new FileIntoActionCommandParser(this));
+        registry.register(ActionCommand.Commands.SET.getJsonName(), new SetActionCommandParser(this));
+
 
         registerService(ActionCommandParserRegistry.class, registry);
         trackService(ActionCommandParserRegistry.class);
