@@ -200,7 +200,7 @@ public abstract class AbstractMailFilterServlet extends HttpServlet {
              */
             final AbstractAction action = getAction();
             response.setData(action.action(request));
-        } catch (final OXException e) {
+        } catch (OXException e) {
             if (SessionExceptionCodes.hasPrefix(e)) {
                 LOG.debug("", e);
             } else {
@@ -216,7 +216,7 @@ public abstract class AbstractMailFilterServlet extends HttpServlet {
         resp.setContentType(CONTENTTYPE_JAVASCRIPT);
         try {
             ResponseWriter.write(response, resp.getWriter(), localeFrom(session));
-        } catch (final JSONException e) {
+        } catch (JSONException e) {
             LOG.error("Error while writing JSON.", e);
             sendError(resp);
         } catch (OXException e) {
@@ -270,7 +270,7 @@ public abstract class AbstractMailFilterServlet extends HttpServlet {
             request.setBody(com.openexchange.ajax.AJAXServlet.getBody(req));
             final AbstractAction action = getAction();
             response.setData(action.action(request));
-        } catch (final OXException e) {
+        } catch (OXException e) {
             if (SessionExceptionCodes.hasPrefix(e)) {
                 LOG.debug("", e);
             } else {
@@ -286,7 +286,7 @@ public abstract class AbstractMailFilterServlet extends HttpServlet {
         resp.setContentType(CONTENTTYPE_JAVASCRIPT);
         try {
             ResponseWriter.write(response, resp.getWriter(), localeFrom(session));
-        } catch (final JSONException e) {
+        } catch (JSONException e) {
             LOG.error("Error while writing JSON.", e);
             sendError(resp);
         } catch (OXException e) {
