@@ -51,20 +51,25 @@ package com.openexchange.jsieve.export;
 
 import java.net.URI;
 import java.net.URISyntaxException;
+import java.util.Optional;
 import javax.mail.internet.idn.IDNA;
 import com.openexchange.config.lean.LeanConfigurationService;
 import com.openexchange.exception.OXException;
-import com.openexchange.groupware.ldap.User;
-import com.openexchange.groupware.ldap.UserStorage;
+import com.openexchange.mailaccount.MailAccount;
+import com.openexchange.mailaccount.MailAccountStorageService;
 import com.openexchange.mailfilter.Credentials;
 import com.openexchange.mailfilter.exceptions.MailFilterExceptionCode;
+import com.openexchange.mailfilter.internal.CircuitBreakerInfo;
 import com.openexchange.mailfilter.properties.CredentialSource;
 import com.openexchange.mailfilter.properties.LoginType;
 import com.openexchange.mailfilter.properties.MailFilterProperty;
 import com.openexchange.mailfilter.properties.PasswordSource;
 import com.openexchange.mailfilter.services.Services;
+import com.openexchange.metrics.MetricService;
 import com.openexchange.tools.net.URIDefaults;
 import com.openexchange.tools.net.URIParser;
+import com.openexchange.user.User;
+import com.openexchange.user.UserService;
 
 /**
  * {@link SieveHandlerFactory}
@@ -76,23 +81,27 @@ public final class SieveHandlerFactory {
     /**
      * Connect to the Sieve server and return a handler
      *
-     * @param creds credentials
-     * @return a sieve handler
+     * @param creds The credentials
+     * @param optionalCircuitBreaker The optional breaker for mail filter access
+     * @param optionalMetricService The optional metric service to measure request/error rate
+     * @return A sieve handler
      * @throws OXException
      */
-    public static SieveHandler getSieveHandler(Credentials creds) throws OXException {
-        return getSieveHandler(creds, false);
+    public static SieveHandler getSieveHandler(Credentials creds, Optional<CircuitBreakerInfo> optionalCircuitBreaker, Optional<MetricService> optionalMetricService) throws OXException {
+        return getSieveHandler(creds, optionalCircuitBreaker, optionalMetricService, false);
     }
 
     /**
      * Connect to the Sieve server and return a handler
      *
-     * @param creds credentials
+     * @param creds The credentials
+     * @param optionalCircuitBreaker The optional breaker for mail filter access
+     * @param optionalMetricService The optional metric service to measure request/error rate
      * @param onlyWelcome <code>true</code> if only server's welcome message is of interest; otherwise <code>false</code> for full login round-trip
-     * @return a sieve handler
+     * @return A sieve handler
      * @throws OXException
      */
-    public static SieveHandler getSieveHandler(Credentials creds, boolean onlyWelcome) throws OXException {
+    public static SieveHandler getSieveHandler(Credentials creds, Optional<CircuitBreakerInfo> optionalCircuitBreaker, Optional<MetricService> optionalMetricService, boolean onlyWelcome) throws OXException {
         LeanConfigurationService mailFilterConfig = Services.getService(LeanConfigurationService.class);
 
         int userId = creds.getUserid();
@@ -102,9 +111,10 @@ public final class SieveHandlerFactory {
         int sievePort;
         String sieveServer;
         User user = null;
+        MailAccount mailAccount = null;
         {
-            String sCredSrc = mailFilterConfig.getProperty(userId, contextId, MailFilterProperty.loginType);
-            LoginType loginType = LoginType.loginTypeFor(sCredSrc);
+            String sLoginType = mailFilterConfig.getProperty(userId, contextId, MailFilterProperty.loginType);
+            LoginType loginType = LoginType.loginTypeFor(sLoginType);
 
             switch (loginType) {
                 case GLOBAL:
@@ -119,14 +129,15 @@ public final class SieveHandlerFactory {
                     if (user.isGuest()) {
                         throw MailFilterExceptionCode.INVALID_USER_SPECIFIED.create();
                     }
-                    String mailServerURL = user.getImapServer();
+                    mailAccount = getMailAccount(userId, contextId);
+                    String mailServerURL = mailAccount.getMailServer();
                     try {
                         URI uri = URIParser.parse(IDNA.toASCII(mailServerURL), URIDefaults.IMAP);
                         if (null == uri) {
                             throw MailFilterExceptionCode.UNABLE_TO_EXTRACT_SIEVE_SERVER_URI.create();
                         }
                         sieveServer = uri.getHost();
-                    } catch (final URISyntaxException e) {
+                    } catch (URISyntaxException e) {
                         throw MailFilterExceptionCode.NO_SERVERNAME_IN_SERVERURL.create(e, mailServerURL);
                     }
                     sievePort = getPort(mailFilterConfig, userId, contextId);
@@ -150,17 +161,17 @@ public final class SieveHandlerFactory {
         CredentialSource credentialSource = CredentialSource.credentialSourceFor(sCredSrc);
         switch (credentialSource) {
             case IMAP_LOGIN: {
-                String authname = getUser(creds, user).getImapLogin();
-                return newSieveHandlerUsing(sieveServer, sievePort, creds.getUsername(), authname, getRightPassword(mailFilterConfig, creds), authEnc, creds.getOauthToken(), creds.getUserid(), creds.getContextid());
+                String authname = getMailAccount(userId, contextId, mailAccount).getLogin();
+                return newSieveHandlerUsing(sieveServer, sievePort, creds.getUsername(), authname, getRightPassword(mailFilterConfig, creds), authEnc, creds.getOauthToken(), optionalCircuitBreaker, optionalMetricService, userId, contextId);
             }
             case MAIL: {
                 String authname = getUser(creds, user).getMail();
-                return newSieveHandlerUsing(sieveServer, sievePort, creds.getUsername(), authname, getRightPassword(mailFilterConfig, creds), authEnc, creds.getOauthToken(), creds.getUserid(), creds.getContextid());
+                return newSieveHandlerUsing(sieveServer, sievePort, creds.getUsername(), authname, getRightPassword(mailFilterConfig, creds), authEnc, creds.getOauthToken(), optionalCircuitBreaker, optionalMetricService, userId, contextId);
             }
             case SESSION:
                 // fall-through
             case SESSION_FULL_LOGIN:
-                return newSieveHandlerUsing(sieveServer, sievePort, creds.getUsername(), creds.getAuthname(), getRightPassword(mailFilterConfig, creds), authEnc, creds.getOauthToken(), creds.getUserid(), creds.getContextid());
+                return newSieveHandlerUsing(sieveServer, sievePort, creds.getUsername(), creds.getAuthname(), getRightPassword(mailFilterConfig, creds), authEnc, creds.getOauthToken(), optionalCircuitBreaker, optionalMetricService, userId, contextId);
             default:
                 throw MailFilterExceptionCode.NO_VALID_CREDSRC.create();
         }
@@ -168,25 +179,27 @@ public final class SieveHandlerFactory {
 
     /**
      * Creates an new {@link SieveHandler} with the specified properties
-     * 
+     *
      * @param host The host
      * @param port The port
-     * @param userName The username
+     * @param userName The user name
      * @param authName The authentication name
      * @param password The password
      * @param authEncoding The authentication encoding
-     * @param oauthToken The oauth token
+     * @param oauthToken The OAuth token
+     * @param optionalCircuitBreaker The optional breaker for mail filter access
+     * @param optionalMetricService The optional metric service to measure request/error rate
      * @param userId The user identifier
      * @param contextId The context identifier
      * @return The {@link SieveHandler}
      */
-    private static SieveHandler newSieveHandlerUsing(String host, int port, String userName, String authName, String password, String authEncoding, String oauthToken, int userId, int contextId) {
-        return new SieveHandler(null == userName ? authName : userName, authName, password, host, port, authEncoding, oauthToken, userId, contextId);
+    private static SieveHandler newSieveHandlerUsing(String host, int port, String userName, String authName, String password, String authEncoding, String oauthToken, Optional<CircuitBreakerInfo> optionalCircuitBreaker, Optional<MetricService> optionalMetricService, int userId, int contextId) {
+        return new SieveHandler(null == userName ? authName : userName, authName, password, host, port, authEncoding, oauthToken, optionalCircuitBreaker, optionalMetricService, userId, contextId);
     }
 
     /**
      * Get the user
-     * 
+     *
      * @param credentials The {@link Credentials}
      * @return the user
      * @throws OXException
@@ -197,7 +210,7 @@ public final class SieveHandlerFactory {
 
     /**
      * Get the user
-     * 
+     *
      * @param creds The {@link Credentials}
      * @param user The optional {@link User}
      * @return the user
@@ -208,16 +221,45 @@ public final class SieveHandlerFactory {
             return user;
         }
 
-        User storageUser = UserStorage.getInstance().getUser(creds.getUserid(), creds.getContextid());
+        User storageUser = Services.requireService(UserService.class).getUser(creds.getUserid(), creds.getContextid());
         if (null == storageUser) {
-            throw MailFilterExceptionCode.INVALID_CREDENTIALS.create("Could not get a valid user object for uid " + creds.getUserid() + " and contextid " + creds.getContextid());
+            throw OXException.general("Could not get a valid user object for uid " + creds.getUserid() + " and contextid " + creds.getContextid());
         }
         return storageUser;
     }
 
+    /**
+     * Get the primary mail account for given user
+     *
+     * @param userId The user identifier
+     * @param contextId The context identifier
+     * @return The mail account
+     * @throws OXException
+     */
+    private static MailAccount getMailAccount(int userId, int contextId) throws OXException {
+        return getMailAccount(userId, contextId, null);
+    }
+
+    /**
+     * Get the primary mail account for given user
+     *
+     * @param userId The user identifier
+     * @param contextId The context identifier
+     * @param mailAccount The optional mail account
+     * @return The mail account
+     * @throws OXException
+     */
+    private static MailAccount getMailAccount(int userId, int contextId, MailAccount mailAccount) throws OXException {
+        if (null != mailAccount) {
+            return mailAccount;
+        }
+
+        return Services.requireService(MailAccountStorageService.class).getDefaultMailAccount(userId, contextId);
+    }
+
     /**
      * Get the port from the configuration service
-     * 
+     *
      * @param mailFilterConfig The {@link MailFilterConfigurationService}
      * @param userId The user identifier
      * @param contextId The context identifier
@@ -227,7 +269,7 @@ public final class SieveHandlerFactory {
     private static int getPort(LeanConfigurationService mailFilterConfig, int userId, int contextId) throws OXException {
         try {
             return mailFilterConfig.getIntProperty(userId, contextId, MailFilterProperty.port);
-        } catch (final RuntimeException e) {
+        } catch (RuntimeException e) {
             throw MailFilterExceptionCode.PROPERTY_ERROR.create(e, MailFilterProperty.port.getFQPropertyName());
         }
     }
