@@ -494,6 +494,31 @@ public final class SieveTextFilter {
         return newRulesList;
     }
 
+    /**
+     * Adds the proper require commands according to the actions used in the command within.
+     * @param rule The rule to add the require command to
+     */
+    public void addRequired(Rule rule) {
+        final Set<String> requires = new HashSet<String>();
+        if (null == rule.getRequireCommand()) {
+            final ArrayList<Command> commands = rule.getCommands();
+            if (!rule.isCommented() && null != commands) {
+                for (final Command command : commands) {
+                    final Set<String> required = command.getRequired();
+                    requires.addAll(required);
+                }
+            }
+            if (!requires.isEmpty()) {
+                final ArrayList<ArrayList<String>> arrayList = new ArrayList<ArrayList<String>>();
+                arrayList.add(new ArrayList<String>(requiredList(requires)));
+                ArrayList<Command> cmds = new ArrayList<>();
+                cmds.add(new RequireCommand(arrayList));
+                cmds.addAll(rule.getCommands());
+                rule.setCommands(cmds);
+            }
+        }
+    }
+
     private static Set<String> requiredList(final Set<String> requires) {
         if (requires.contains("imapflags") && requires.contains("imap4flags")) {
             // Prefer "imap4flags" if both supported
@@ -518,7 +543,7 @@ public final class SieveTextFilter {
             int minDiff = Integer.MAX_VALUE;
             Rule rightRule = null;
             for (final Rule rule : rules) {
-                final int abs = getPosDif(rule.getLinenumber() - ruleComment.getLine());
+                final int abs = getPosDif (rule.getLinenumber() - ruleComment.getLine());
                 if (abs < minDiff) {
                     minDiff = abs;
                     rightRule = rule;
@@ -620,7 +645,7 @@ public final class SieveTextFilter {
         return size;
     }
 
-    private int getPosDif(final int i) {
+    private int getPosDif (final int i) {
         return (i > 0) ? i : Integer.MAX_VALUE;
     }
 
